/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.SubmitButton;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIChangeParticipant implements CGICommand
{

   private void changeParticipantPseudonym(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException
   {
      // 0 = select event, 1 = select participant, 2 = select pseudonym, 3 = commit
      if (null != params.get("reset")) {
         umpirecgi.currentAuth.unsetData("cpp_event");
         umpirecgi.currentAuth.unsetData("cpp_assassin");
      }
      int pass = 0;
      Event e = null;
      if (null !=  params.get("1") && !"".equals(params.get("1")))
         e = Event.getByID(Integer.parseInt((String) params.get("1")));
      if (null != umpirecgi.currentAuth.getData("cpp_event"))
         e = Event.getByID(Integer.parseInt(umpirecgi.currentAuth.getData("cpp_event")));
      if (null != params.get("event")) 
         e = (Event) params.get("event");
      Assassin a = null;
      if (null !=  params.get("2") && !"".equals(params.get("2")))
         a = e.getParticipant(Integer.parseInt((String) params.get("2")));
      if (null != params.get("assassin")) 
         a = (Assassin) params.get("assassin");
      if (null != umpirecgi.currentAuth.getData("cpp_assassin"))
         a = Assassin.getByAid(Integer.parseInt(umpirecgi.currentAuth.getData("cpp_assassin")));
      Pseudonym p = null;
      if (null != params.get("pseudonym"))
         p = (Pseudonym) params.get("pseudonym");

      cgi.out("<h2>Change Pseudonym used in an Event</h2>");
      HTMLForm form = new HTMLForm("umpire.cgi?change-participant-pseudonym", "Change Pseudonym");

      if (null == e) pass = 0;
      else if (null == a) pass = 1;
      else if (null == p) pass = 2;
      else pass = 3;

      switch (pass) {
         case 0:
            cgi.out("<p>");
            cgi.out("  Select Event:");
            cgi.out("</p>");
            List l = Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(new CanonicalEntityField("event","Event:",l,(CanonicalEntity) null));
            form.display(cgi);
            break;
         case 1:
            Collection participants = new Vector();
            if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               List pa = umpirecgi.currentAuth.getPlayerViewer().getAssassins();
               if (Debug.debug) Debug.print(umpirecgi.class, "got player assassins:"+pa);
               Collection ea = e.getParticipants().values();
               if (Debug.debug) Debug.print(umpirecgi.class, "got participants:"+ea);
               Iterator i = pa.iterator();
               while (i.hasNext()) {
                  Object o = i.next();
                  if (ea.contains(o)) participants.add(o);
               }
               if (Debug.debug) Debug.print(umpirecgi.class, "got intersection:"+participants);
               if (participants.size() == 0) {
                  cgi.out("<div class=\"errormessage\">");
                  cgi.out("  <p><b>Insufficient Privileges</b></p>");
                  cgi.out("  <p>You are not a participant in this event!</p>");
                  cgi.out("</div>");
                  return;
               }
            } else
               participants = e.getParticipants().values();
            cgi.out("<p>");
            cgi.out("  Select Participant:");
            cgi.out("</p>");
            form.addField(new DisplayField("Event:", e.getPrintable()));
            umpirecgi.currentAuth.storeData("cpp_event", ""+e.getEid());
            form.addField(new CanonicalEntityField("assassin","Participant:",
                     participants,(CanonicalEntity)null));
            form.addField(new SubmitButton("reset", "Reset"));
            form.display(cgi);
            break;
         case 2:
            cgi.out("<p>");
            cgi.out("  Select Pseudonym:");
            cgi.out("</p>");
            form.addField(new DisplayField("Event:", e.getPrintable()));
            umpirecgi.currentAuth.storeData("cpp_event", ""+e.getEid());
            umpirecgi.currentAuth.storeData("cpp_assassin", ""+a.getAid());
            form.addField(new DisplayField("Participant:",a.getPrintable()));
            if (Debug.debug) Debug.print(umpirecgi.class, "got participant:"+a);

            LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
            lsb.registerInterest(a);
            Map myState = null;
            try {
               myState = lsb.generateState();
            } catch (StateBuilderException sbe) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("  <p><b>Internal Error</b></p>");
               cgi.out("  <p>State Building failed! Very bad!!</p>");
               cgi.out("</div>");
               cgi.flush();
               throw new RuntimeException();
            }
            if (Debug.debug) Debug.print(umpirecgi.class, "got possible pseuds:"+((AssassinState) myState.get(a)).getPseudonyms());
            int n = 0;
            Iterator i = e.getParticipants().keySet().iterator();
            while (i.hasNext()) {
               n = ((Integer) i.next()).intValue();
               if (e.getParticipant(n).equals(a)) break;
            }
            p = e.getParticipantPseudonym(n);
            if (Debug.debug) Debug.print(umpirecgi.class, "got current pseud:"+p);
            form.addField(new CanonicalEntityField("pseudonym","Pseudonym:",((AssassinState) myState.get(a)).getPseudonyms(), p));
            form.addField(new CheckBox("nopseudonym", "Use Real Name:", null==p));
            form.addField(new SubmitButton("reset", "Reset"));
            form.display(cgi);
            break;
         case 3:
            try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 
            if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
               if (!a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
                  cgi.out("<div class=\"errormessage\">");
                  cgi.out("  <p><b>Insufficient Privileges</b></p>");
                  cgi.out("  <p>I'm sorry, only the umpire can change other people's pseudonyms.</p>");
                  cgi.out("</div>");
                  return;
               }
            }
            cgi.out("<div class=\"donemessage\">");
            cgi.out("<p><b>Pseudonym Successfully Changed</b></p>");
            cgi.out("<p>&nbsp;</p>");
            cgi.out("</div>");
            n = 0;
            i = e.getParticipants().keySet().iterator();
            while (i.hasNext()) {
               n = ((Integer) i.next()).intValue();
               if (e.getParticipant(n).equals(a)) break;
            }
            if ("on".equals(params.get("nopseudonym")))
               e.setParticipantPseudonym(n, null);
            else
               e.setParticipantPseudonym(n, p);
            Event.commitAll();
            umpirecgi.currentAuth.unsetData("cpp_event");
            umpirecgi.currentAuth.unsetData("cpp_assassin");
            break;
      }
   }
   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException { changeParticipantPseudonym(cgi, plugins, params); } 
   public String getName() { return "Change Participant Pseudonym"; }
   public String getDescription() { return "Changes the pseudonym that an assassin is using in an event"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }
}

