/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */
package org.ucam.srcf.assassins;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.TimeZone;
import java.util.Vector;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.xml.ColourFilter;
import org.ucam.srcf.assassins.xml.Config;

public class Tools
{   
   public static final String NAME = "CUAG AutoUmpire";
   public static final String VERSION = "0.2";
   public static final String RELEASENAME = "Cone";
   public static final String VERSIONSTRING = NAME + " v"+VERSION + " ("+RELEASENAME+")";

   public static final String defaultConfigFile = "config.xml";
   public static final String[] configSearchPath = {
      ".",
      "../config"
   };
 
   static DateFormat df = new SimpleDateFormat("d-M-yyyy H:mm:ss");
   static DateFormat fTim = new SimpleDateFormat("d MMM, HH:mm a");
   static DateFormat fDay = new SimpleDateFormat("EEEE, d MMMM");
   static DateFormat fToD = new SimpleDateFormat("HH:mm a");
   static TimeZone tz = TimeZone.getTimeZone("Europe/London");
   static {
      df.setTimeZone(tz);
      fTim.setTimeZone(tz);
      fDay.setTimeZone(tz);
      fToD.setTimeZone(tz);
   }

   private static long nowtime = 0;
   public static long getNowTime() { return nowtime; }
   static void setNowTime(long time) { nowtime = time; }
   static void setNowTime(Date time) { nowtime = time.getTime(); }
   static void setNowTime(String time) throws ParseException { nowtime = unFormatTime(time); }

   /**
    * Locates the config file.
    * Searchs configSearchPath for deafultConfigFile
    * @return path to first config file found in the search path, or the
    * path of the config file would have in the first search path if it 
    * cannot be found.
    */
   public static String findConfig() {
      for(int i=0; i<configSearchPath.length; i++) {
         String filename = configSearchPath[i] + "/" + defaultConfigFile;
         if((new File(filename)).exists())
            return filename;
      }
      /* If we can't find it pass back the one in the firt search path */
      return configSearchPath[0]+"/"+defaultConfigFile;
   }
   
   public static long unFormatTime(String timestr) throws ParseException
   {
      /*StringTokenizer tokenizer = new StringTokenizer(timestr,":- ");
      Date x;
      try {
         int day = Integer.parseInt(tokenizer.nextToken());
         int month=Integer.parseInt(tokenizer.nextToken());
         int year= Integer.parseInt(tokenizer.nextToken());
         if (year<30) year +=100;
         int hour= Integer.parseInt(tokenizer.nextToken());
         int minute=Integer.parseInt(tokenizer.nextToken());
         int second=Integer.parseInt(tokenizer.nextToken());
         if (Debug.debug) Debug.print(Tools.class, "Parsed date" + year + " " + month + " " + day + " " + hour + " " + minute + " " + second);
       x = new Date(year,month,day,hour,minute,second);
      } catch (Exception e) {
         return -1;
      }*/
      SimpleDateFormat d = new SimpleDateFormat("d-M-yyyy H:mm:ss");
      if (Debug.debug) Debug.print(Tools.class, "Parsing " + timestr);
      Date x = d.parse(timestr);
      //      if (Debug.debug) Debug.print(Tools.class, "Parsed as "+x.toGMTString()); 
      return x.getTime();
   }

   /** WARNING!!! HUGE HACK AHEAD---those of a sensitive disposition should look away now */
   static long time_fudge_factor = 1000*60*60;

   public static String formatTime(long time)
   {
      return df.format(new Date(time+time_fudge_factor));
   }

   public static String prettyDateAndTime(long time)
   {
      return fTim.format(new Date(time+time_fudge_factor));
   }
   
   public static String prettyDate(long time)
   {
      return fDay.format(new Date(time+time_fudge_factor));
   }
   
   public static String prettyTime(long time)
   {
      return fToD.format(new Date(time+time_fudge_factor));
   }

   public static String escape(String in)
   {
      if (Debug.debug) Debug.print(Tools.class, "escaping: "+in);
      String out = "";
      for (int i = 0; i < in.length(); i++) {
         if (in.charAt(i) == '\'') out += '\\';
         out += in.charAt(i);
      }
      return out;
   }

   public static String escapeXML(String in)
   {
      if (in == null) return null;
      String out = "";
      for (int i = 0; i < in.length(); i++)
      {
         switch (in.charAt(i))
         {
            case '&':
               // preserve existing entities
               if( i+1 == in.length()) { out += "&amp;"; break; }
               String s = "";
               i++;
               while ( i < in.length() && "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".indexOf(in.charAt(i)) > 0 ) 
                  s += in.charAt(i++);
               if (i == in.length()) out += "&amp;" + s;
               else if (';' == in.charAt(i)) out += "&"+s+";";
               else out += "&amp;" + s + in.charAt(i);
               break;
            case '<':
               out += "&lt;"; break;
            case '>':
               out += "&gt;"; break;
            case '"':
               out += "&quot;"; break;
            default:
               out += in.charAt(i);
         }
      }
      return out;
   }
   
   /**
    * Method to inverse the result of escapeXML so that unescapeXML(escapeXML(s))==s
    * @param in
    * @return in with characters returned as entities
    */
   public static String unescapeXML(String in) {
       if (in==null) return null;
       String out = "";
       for (int i=0; i<in.length(); i++) {
           if (in.charAt(i)=='&') {
               String s = "&";
               i++;
               while ( i < in.length() && "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789#".indexOf(in.charAt(i)) > 0 ) 
                  s += in.charAt(i++);
               if (in.charAt(i)!=';') return in;
               s+=';';
               if (s.equals("&lt;")) out = out + "<";
               if (s.equals("&gt;")) out = out + ">";
               if (s.equals("&quot;")) out=out+"\"";
               if (s.equals("&amp;")) out=out+"&";
           } else {
               out+=in.charAt(i);
           }
       }
       return out;
   }

   public static void exec(String[] args)
   {
      exec(args, System.err);
   }

   public static void exec(String[] args, OutputStream errors)
   {
      if (null == errors) exec(args, System.err);
      else {
         PrintWriter err = new PrintWriter(errors);
         exec(args, err);
      }
   }
   
   public static void exec(String[] args, PrintWriter err)
   {
      exec(args, err, null);
   }
   public static void exec(String[] args, PrintWriter err, PrintWriter out)
   {
      if (Debug.debug) {
         if (null == args) Debug.print(Tools.class, "Warning! arguments are null!");
         else {
            String arglist = "{";
            for (int i=0; i < args.length; i++)
               arglist += args[i]+",";
            arglist += "}";
            Debug.print(Tools.class, "executing "+arglist);
         }
      }
      try {
         Process p = Runtime.getRuntime().exec(args);
         BufferedReader stderr = new BufferedReader(new InputStreamReader(p.getErrorStream()));
         String s;
         if (null != out) {
            BufferedReader stdout = new BufferedReader(new InputStreamReader(p.getInputStream()));
            while (null != (s = stdout.readLine()))
               out.println(s);
         }
         while (null != (s = stderr.readLine()))
            err.println(s);
         p.waitFor();
      } 
      catch (InterruptedException Ie) { Ie.printStackTrace(); }
      catch (IOException IOe) { IOe.printStackTrace(); }
   }
 
   public static String replace(String x, Map withBy) {
      for (int i=0; i<x.length(); i++) {
         if (x.charAt(i)!='[') continue;
         int nextRightBracket = x.indexOf(']', i);
         int nextLeftBracket = x.indexOf('[', i+1);
         if (nextRightBracket<0) continue; // If there are no more ]s (mismatched)
         if (nextLeftBracket<nextRightBracket && nextLeftBracket>0) continue;
         String y = "";
         char t;
         int j =0;
         while ((t = x.charAt(i+(++j)))!=-1 && ']'!=t) y+=t;
         String repl = (String) ((null==withBy.get(y))?"["+y+"]":withBy.get(y));
         x = x.substring(0,i)+repl+x.substring(i+j+1, x.length());
         i+=repl.length();
      }
      return x;
   }

	/**
	 * Replace at signs in email by &lt;at /&gt; which is then
	 * rewritten by xsl
	 */
	public static String obfuscateEmail(String toObfuscate) {
		/*
		 * Sadly this isn't implemented yet in libgcj
		 * return toObfuscate.replaceAll("@", "<at />");
		 */
		if(toObfuscate==null) return null;
		String ret=new String(toObfuscate);
		for(int i=0; i<ret.length(); i++) {
			if(ret.charAt(i)!='@') continue;
			ret = ret.substring(0,i) + "<at />" + ret.substring(i+1,ret.length());
			i+=5;
		}
		
		return ret;
	}

   public static Map viewerifyStateMap(Map in)
   {
      Map out = new HashMap();
      Iterator i = in.keySet().iterator();
      while (i.hasNext())
      {
         Assassin key = (Assassin) i.next();
         AssassinState value = (AssassinState) in.get(key);
         if (Debug.debug)
         {
            if (null == key || null == value)
               Debug.print(Tools.class, "null key or value in viewerifyStateMap");
         }
         AssassinViewer av = new AssassinViewer(key);
         AssassinStateViewer asv = new AssassinStateViewer(value);
         out.put(av, asv);
      }
      return out;
   }

   /**
    * Takes a list of Canonical Entities and returns a list of viewers.
    * <b>Undefined results if CanonicalEntites are immutable</b>.
    * List returned is actually a vector
    */
   public static List viewerifyList(List in) {
      List out = new Vector();
      
      Iterator i = in.iterator();
      while(i.hasNext()) {
         CanonicalEntity c = (CanonicalEntity) i.next();
         out.add(c.getViewer());
      }
      
      return out;
   }

   static String getEmailText(Map state, Map plugins, boolean force, String manualText) throws AssassinSQLException, ParseException
   {
      if (Debug.debug) Debug.print(Tools.class,"sendEmails("+state+","+plugins+","+force+","+manualText+")");
      String emails = "";
      String from = Config.getConfig("vanilla", "from-address");
      String subject = Config.getConfig("vanilla", "email-subject");
      Iterator i = state.values().iterator();
      Map statev = viewerifyStateMap(state);
      while(i.hasNext())
      {
         AssassinState as = (AssassinState) i.next();
         if (Debug.debug) Debug.print(Tools.class,""+as);
         if (!as.isAlive()) continue;
         if (!as.isActive()) continue;
         boolean needsemail = force;
         Iterator j = Plugin.getPluginIterator(plugins);
         AssassinStateViewer asv = new AssassinStateViewer(as);
         while(j.hasNext() && !needsemail) 
            needsemail |= ((Plugin) j.next()).playerNeedsEmail(asv);


         if (needsemail) {
            Player p = as.getAssassin().getPlayer();
            // get text
            String emailText = "Your Details:\n";
            emailText += "           Name: "+p.getName()+"\n";
            emailText += "        Address: "+p.getAddress()+"\n";
            emailText += "        College: "+p.getCollege()+"\n";
            emailText += "   Water Status: "+p.getWaterStatus()+"\n";
            emailText += "          Notes: "+p.getNotes()+"\n";

            j = Plugin.getPluginIterator(plugins);
            while(j.hasNext()) {
               String emailadd = ((Plugin) j.next()).generateEmail(asv, statev);
               if (null != emailadd) emailText += "\n----------------\n" + emailadd;
            }

            // print text & headers
            emails += "MAIL FROM:" + from + "\n";
            emails += "RCPT TO:" + p.getEmail() + "\n";
            emails += "DATA\n";
            emails += "From: "+ from +"\n";
            emails += "Subject: "+ subject +"\n";
            emails += manualText + "\n";
            emails += emailText + "\n\n.\n";
         }
      }
      emails += "QUIT\n";
      return emails;
   }

   public static String getColourForAssassin(Map plugins, AssassinState as, String name)
   {
      String colourClass = getColourClassForAssassin(plugins, as);
      if (null == colourClass) return "unknown";
      int range = Config.getConfigInt("vanilla", "colourcount-" + colourClass);
      if (range == 0) return "unknown";
      int suffix = 1 + (new Random(name.hashCode())).nextInt(range);
      return colourClass + suffix;
   }
   
   public static String getColourClassForAssassin(Map plugins, AssassinState as)
   {
      Map classes = Config.getColourFilters();
      Iterator i = classes.keySet().iterator();
      if (Debug.debug) Debug.print(PageCreator.class, "finding match");
      while (i.hasNext())
      {
         String classKey = (String) i.next();
         ColourFilter cf = (ColourFilter) classes.get(classKey);
         if (Debug.debug) Debug.print(PageCreator.class, " trying " + classKey);
         if (cf.matchAssassinState(plugins, as)) return classKey;
      }
      if (Debug.debug) Debug.print(PageCreator.class, " failure");
      return null;
   }

    /**
     * Converts a byte array into a hex string
     */
    public static String toHexStr(byte[] bs)
    {
       String h = "";
       for (int k=0; k<20; k++) {
          int[] a = new int[2];
          a[0] = bs[k] & 0x0F;
          a[1] = (bs[k] >> 4) & 0x0F;
          for (int j=0; j<2; j++) switch (a[j]) {
            case 0x0: h+="0"; break;
            case 0x1: h+="1"; break;
            case 0x2: h+="2"; break;
            case 0x3: h+="3"; break;
            case 0x4: h+="4"; break;
            case 0x5: h+="5"; break;
            case 0x6: h+="6"; break;
            case 0x7: h+="7"; break;
            case 0x8: h+="8"; break;
            case 0x9: h+="9"; break;
            case 0xa: h+="a"; break;
            case 0xb: h+="b"; break;
            case 0xc: h+="c"; break;
            case 0xd: h+="d"; break;
            case 0xe: h+="e"; break;
            case 0xf: h+="f"; break;
          }
       }
       return h;
    }
}
