/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.lemon;

import java.text.ParseException;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;

import org.ucam.srcf.assassins.AssassinActivation;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.AuthLevel;
import org.ucam.srcf.assassins.ConfigProperty;
import org.ucam.srcf.assassins.EventViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.StateChange;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;
import org.ucam.srcf.assassins.Assassin;
import org.ucam.srcf.assassins.PluginCommand;
import org.ucam.srcf.assassins.TextQuestion;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.Resurrection;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.PluginConnector;
import org.ucam.srcf.assassins.Event;
import org.ucam.srcf.assassins.PlayerViewer;
import org.ucam.srcf.assassins.Timer;
import org.ucam.srcf.assassins.Tools;
import org.ucam.srcf.assassins.umpirecgi;

import org.ucam.srcf.assassins.exceptions.ConfigFileException;

/**
 * Interface for java classes which act as plug-in game modifiers.
 */
public class Lemon extends Plugin
{
   // private long two_hour_hack = 1000*60*60*2; // FIXME, *PLEASE*
   
   public static String getDescription() { return "Plugin to track lists of killers and victims"; };

   /** Start time of game, in millis*/
   long startTime; //--unsure if this is used for anything. amgb2
   
   /**
    * Called whenever the plugin is required to update or sanity-check the
    * current gamestate.
    */
   protected Vector updateState(Map state, Vector qs, EventViewer et) 
		throws AssassinSQLException, ParseException 
	{ 
		if(Debug.debug) Debug.print(this, "Sanity-checking kill lists");

      /* Run the entire game and validate the scores */
      fixupEverything();
	return null; 
   }

   private void fixupEverything() throws AssassinSQLException
   {
      if (Debug.debug) Debug.print(this, "Entering fixupEverything()");
      // get all the kill/death statechanges
      //
      // what I am about to do is hideous. however the time handling is so
      // broken at the moment than I don't have a better alternative. I'd
      // like to just get them *all*, but there's no method that doesn't
      // take a time.
      List sclist = StateChange.getByClass(Death.class);

      if (Debug.debug) Debug.print(this, "Found " + sclist.size() + "death statechanges");

      // make some empty lists
      List allAssassins = Assassin.getAllAssassinViewers();
      Map killcounts = new HashMap();
      Map deathcounts = new HashMap();
      
      // iterate through statechanges
      Iterator sci = sclist.iterator();
      while (sci.hasNext())
      {
         Integer previous;
         Death d = (Death) (sci.next());
         EventViewer e = d.getEventViewer();
         if (!e.isActive()) continue;
         AssassinViewer killer = (AssassinViewer) (d.getKiller());
         AssassinViewer victim = (AssassinViewer) (d.getVictim());

         if (Debug.debug) Debug.print(this, "Processing: " + killer.getPlayerViewer().getName() + " kills " + victim.getPlayerViewer().getName());
         
         // for each one, calculate the new lists for the killer ...
         Map killerMap = (Map) killcounts.get(killer);
         if (null == killerMap) killerMap = new HashMap();
         previous = (Integer) (killerMap.get(victim));
         if (null == previous)
            killerMap.put(victim, new Integer(1));
         else
            killerMap.put(victim, new Integer(previous.intValue() + 1));
         killcounts.put(killer, killerMap);
         // ... and the victim
         Map victimMap = (Map) deathcounts.get(victim);
         if (null == victimMap) victimMap = new HashMap();
         previous = (Integer) (victimMap.get(killer));
         if (null == previous)
            victimMap.put(killer, new Integer(1));
         else
            victimMap.put(killer, new Integer(previous.intValue() + 1));
         deathcounts.put(victim, victimMap);
         
         // delete all lemon scs on the event
         List children = e.getStateChanges();
         Iterator chit = children.iterator();
         while (chit.hasNext())
         {
            StateChange child = (StateChange) chit.next();
            if (child instanceof PluginStateChange)
            {
               if (((PluginStateChange)child).getPluginName() == getPluginName())
               {
                  vanilla.removeStateChange(e, child);
               }
            }
         }
         
         // add new ones
         vanilla.addStateChange(e, new PluginStateChange(killer, this, "kill-list", formatAssassinNumberMap(killerMap)));
         vanilla.addStateChange(e, new PluginStateChange(victim, this, "death-list", formatAssassinNumberMap(victimMap)));
      }

      if (Debug.debug) Debug.print(this, "Leaving fixupEverything()");
   }
   
   protected Vector stateChangeAdded(StateChange sc, Map state, Vector qs) throws AssassinSQLException
   {
      // do this later. for now, just see if we can make the thing work at all!
      return null;
   }

   private String formatAssassinNumberMap(Map m) throws AssassinSQLException
   {
      boolean first = true;
      String result = "";
      Iterator i = m.keySet().iterator();
      while (i.hasNext())
      {
         AssassinViewer key = (AssassinViewer) i.next();
         Integer value = (Integer) m.get(key);
         String text = key.getPlayerViewer().getName();
         if (value.intValue() > 1)
         {
            text += " (" + value.toString() + " times)";
         }
         if (first)
            first=false;
         else
            result += ", ";
         result += text;
      }
      return result;
   }
   
}
